package edu.georgetown.gucs.dictionary;

import java.io.File;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.Map.Entry;

import edu.georgetown.gucs.tokenizers.TokenizerList;
import edu.georgetown.gucs.utility.*;
import org.slf4j.LoggerFactory;

/**
 * Thread for processing a file to add to a {@linkplain Dictionary}
 *
 * @author Clay Shields
 */
class DictionaryWorker extends Thread {

    private final Dictionary parent;
    private final List<String> tokenizerNames;
    private final FileLister fileLister;
    private final File poison;
    private static final org.slf4j.Logger logger = LoggerFactory.getLogger(Dictionary.class);


    /**
     * Constructor that specifies a file to tokenize and add to an existing dictionary
     *
     * @param fileLister     the file to tokenize and add to this dictionary
     * @param tokenizerNames the list containing the names of the tokenizers to use on this file
     * @param parent         the dictionary to add the file to
     * @param poison         indicates when there are no more files in the queue
     */
    DictionaryWorker(List<String> tokenizerNames, Dictionary parent, FileLister fileLister, File poison) {
        this.tokenizerNames = tokenizerNames;
        this.parent = parent;
        this.fileLister = fileLister;
        this.poison = poison;

    }

    /**
     * Adds a file to an existing dictionary using the given tokenizers
     */
    @Override
    public void run() {
        boolean done = false;

        // Create a tokenizer list from the names of tokens provided
        try {
            TokenizerList tokenizers = null;
            tokenizers = new TokenizerList(tokenizerNames);


            while (!done) {

                //this is error --> takeFile is not a queue
                // process files that come from the fileLister
                File file = fileLister.takeFile();

		 if (file == poison) {
                    done = true;
                    fileLister.putPoison(poison);
                    return;
                }


		logger.info("Got: " + file);
		
                // create a map to hold all the items
                Map<Token, DictionaryEntry> newDictionary = Collections.synchronizedMap(new HashMap<Token, DictionaryEntry>());


                for (Entry<String, List<Token>> splitterEntry : tokenizers.tokenizeFile(file.getAbsolutePath()).entrySet()) {
                    for (Token token : splitterEntry.getValue()) {
                        if (token.name().length() > 0) {

                            // System.out.println("TokenName: " + token.name());

                            if (newDictionary.containsKey(token)) {
                                newDictionary.get(token).incrementFrequencyCount();
                            } else {
                                DictionaryEntry dictEntry = new DictionaryEntry(token);
                                newDictionary.put(token, dictEntry);
                            }
                        }
                    }

                    if (newDictionary.size() > 0) {

                        this.parent.mergeDictionary(newDictionary);

                    } else {
                        logger.warn("DictionaryWorker", "No tokens added to dictionary from " + file);
                    }
                }
            }
        } catch (SdTextException e) {
	    // run() can't throw an SdTextException, so throw a runtime exception and
	    // our other handlers should convert it to an SdTextException
	    throw new RuntimeException (e);
        }
    }
}
