package edu.georgetown.gucs.experiment;

import edu.georgetown.gucs.tokenizers.TokenizerParameters;
import edu.georgetown.gucs.utility.*;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.util.*;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * @author Evan B.
 * @since 6/17/14.
 */
public class Experiment {

    private ExperimentConfReader config;

    //initialized in constructor
    private int trialThreads;
    private String outputFile = null;

    private ArrayList<Trial> threads;

    private List<ExperimentResults> trialResults = new ArrayList<>();

    private Map<String, ArrayList<Pair<Integer, Integer>>> aggregated = new HashMap<>();

    private boolean finished = false;

    TrialParameters poison = new TrialParameters();

    private static final org.slf4j.Logger logger = LoggerFactory.getLogger(Experiment.class);


    //  private final ResultTracker tracker = ResultTracker.getResult();


    /**
     * Constructor that takes a configFile
     *
     * @param configFile configuration options
     */
    public Experiment(String configFile) throws SdTextException {

	System.err.println("In Experiment constructor");

	FileChecker.canRead(configFile,"Experiment","Experiment");
	
        try {
	    System.err.println("Opening config");   
	    config = new ExperimentConfReader(configFile);

	    System.err.println("Setting threads");
            trialThreads = config.getNumTrialThreads();	    
            threads = new ArrayList<>(trialThreads);

	    System.err.println("Exit Experiment Constructor");
            logger.info("[Total Trials] {} ", config.getTotalTrialCount());
        } catch (SdTextException sdtext) {
            throw sdtext;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
    }

    /**
     * public function to run trials
     * Print results to stdOutput
     * OR
     * Save results to File
     */
    public void runExperiment() throws SdTextException {

		System.err.println("In run experiment");
        try{
        logger.info("[Starting trials]");
	System.err.println("calling run trials");
        int runNumber = runTrials();
        } catch (SdTextException sdtext) {
            throw sdtext;
        }catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
    }


    private int runTrials() throws SdTextException {

	System.err.println("In run trials");
	
        int runNumber = 0;

        try {
            logger.info("[Inside runTrials]");

	    System.err.println("1");
            FileLister fileLister = new FileLister(config.getDataSetPath().getAbsolutePath());


            //logger.log(Level.INFO, "[FileLister Size] " + fileLister.size());

            // We also want to wait while the file lister runs, as it is threaded and we don't want concurrency
            // issues
            try {
                while (!fileLister.getLoaded()) {

                    logger.info("[fileLister not loaded, sleep 250MS]");
                    Thread.sleep(250);
                }
            } catch (InterruptedException e) {
                ExceptionHandler.handleException(e);
                logger.info("[Interrupted Exception in fileLister]");
            }

	    System.err.println("About to start trial theads");
            LinkedBlockingQueue<TrialParameters> threadParameterQueue =
                    new LinkedBlockingQueue<>(trialThreads * 10);

            for (int i = 0; i < trialThreads; i++) {

                logger.info("[Starting trial threads]");
                threads.add(i, new Trial(threadParameterQueue, poison, this));

                threads.get(i).setName("Trial thread " + i);
                threads.get(i).setDaemon(true);
                threads.get(i).start();
            }
	    
	    System.err.println("Calling unpackRunTrial");
            unpackRunTrial(fileLister, threadParameterQueue);
	    System.err.println("Returned from unpackRunTrial");

        } catch (SdTextException sdtextE) {
            throw sdtextE;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
        return runNumber;
    }

    public void unpackRunTrial(FileLister fileLister, LinkedBlockingQueue<TrialParameters> threadParameterQueue) throws SdTextException {

        try {
	    System.err.println("In unpackRunTrial with this many trials: " + config.getTotalTrialCount());
            logger.info("[Unpacking Trial Params]");


	    
            Random random = new Random(System.currentTimeMillis());

            for (Double dictionarySampleSize : config.getDictionaryParameters()) {

                logger.info("[Unpacking DictionaryParams " + dictionarySampleSize);

                for (Double sampleSize : config.getSampleParameters()) {

                    logger.info("[Unpacking sample params] {}", sampleSize);

                    for (TokenizerParameters tokenizerParameters : config.getTokenizerParameters()) {

                        logger.info("[tokenizeParams] {} ", tokenizerParameters);

                        for (Pair<Double, Double> idfPair : config.getIDF()) {

                            logger.info("[Unpacking idfPair] {} ", idfPair);

                            for (String fingerprinter : config.getFingerprinter()) {

                                logger.info("[Unpacking fingerprinter] {}", fingerprinter);

                                for (Pair<String, Integer> matcherPair : config.getMatcherVector()) {

                                    logger.info("[Unpacking matcherPair] {} {} ", matcherPair.getA(), matcherPair.getB());

                                    for (String splitter : tokenizerParameters.splitters()) {

                                        logger.info("[Unpacking Splitter] {}", splitter);

                                        for (String mangler : tokenizerParameters.manglers()) {

                                            logger.info("[Unpacking mangler] {}", mangler);

                                            for (int i = 0; i < config.getNumTrials(); i++) {

                                                logger.info("[Creating trialParameter Objects]");
						System.err.println("[Creating trialParameter Objects]");
                                                String matcherName = matcherPair.getA();

                                                int minScore = matcherPair.getB();

                                                TrialParameters trialParameters =
                                                        new TrialParameters(i, config.getDataSetDirectory(),
                                                                fileLister,
                                                                tokenizerParameters.tokenizers(),
                                                                dictionarySampleSize,
                                                                sampleSize, idfPair, fingerprinter, matcherName,
                                                                minScore, mangler, splitter);

                                                trialParameters.setRandom(random);
                                                try {

                                                    threadParameterQueue.put(trialParameters);

                                                    //  experimentID++;
                                                    logger.info("!!++> Adding parameter set, current length is: {} ", threadParameterQueue.size());
						    System.err.println("!!++> Adding parameter set, current length is: {} "+ threadParameterQueue.size());
                                                } catch (InterruptedException e) {
                                                    ExceptionHandler.handleException(e);

                                                    logger.info("[ThreadPareteQueue.put] interrupted exception");
                                                }

                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            logger.info("[Exiting Unpacking]");
            try {
		System.err.println("In goes the poison");
                threadParameterQueue.put(poison);
            } catch (InterruptedException e) {
                ExceptionHandler.handleException(e);
            }


            // Wait for the threads to all terminate

            while (!finished) {
                try {
		    System.err.println("Waiting for threads to finish");
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    ExceptionHandler.handleException(e);
                }


                int finishedThreadCount = 0;
                for (int i = 0; i < trialThreads; i++) {
                    System.err.println("  ---> " + threads.get(i).getState());
                    if (threads.get(i).getState() == Thread.State.valueOf("TERMINATED")) {
                        finishedThreadCount++;
                    }
                }
                if (finishedThreadCount == trialThreads) {
                    finished = true;
                }
            }
        } catch (SdTextException sdtextE) {
            throw sdtextE;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }

        logger.info("Experiment runTrials end");

    }

    /**
     * Change this to use a queue with thread
     *
     * @param results the result of each trial
     *                Removed synchronized
     */
    public synchronized void getResults(ExperimentResults results) {
        // Seperate thread to output results
        // Thread shuts down when main thread is killed
        // Potential memory leak
        ResultTracker.getResult().log(results);

        trialResults.add(results);

    }


    public void graphResults(String fileName) throws SdTextException {
        try {
            JGraph graph = new JGraph(averageTrials(), fileName);
            graph.setTitle("Experiment Results");
            graph.savePlot();
        } catch (SdTextException sdtext) {
            throw sdtext;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
    }

    /**
     * @return a list containing the average score
     * for a trial with x parameters
     */
    private HashMap<String, Pair<Integer, Double>> averageTrials() {


        HashMap<String, ArrayList<ExperimentResults>> totalResults = new HashMap<>();


        for (ExperimentResults avgResult : trialResults) {

            String key = avgResult.getMangler() + ":" + avgResult.getIdfRange().getA() + "_" +
                    avgResult.getIdfRange().getB();

            //change this to reflect IDF Range && Mangler
            if (totalResults.containsKey(key)) {

                totalResults.get(key).add(avgResult);

            } else {
                //add idfRange to the key
                ArrayList<ExperimentResults> trial = new ArrayList<>();
                trial.add(avgResult);

                totalResults.put(key, trial);
            }


        }


        HashMap<String, Pair<Integer, Double>> totalAverages = new HashMap<>();
        for (Map.Entry<String, ArrayList<ExperimentResults>> it : totalResults.entrySet()) {

            Integer temp = 0;
            for (ExperimentResults result : it.getValue()) {

                temp += result.getAverageMatcherScore();

            }
            temp /= it.getValue().size();

            double stdDeviation = stdDeviation(it.getValue(), temp);
            // System.out.println("Mean: " + temp + " Deviation+: " + (stdDeviation + temp) + " Deviation-: " + (temp - stdDeviation) );

            totalAverages.put(it.getKey(), new Pair<>(temp, stdDeviation));

        }


        return totalAverages;
    }


    public double stdDeviation(ArrayList<ExperimentResults> totalAverages, double average) {

        double temp = 0;
        for (ExperimentResults it : totalAverages) {

            temp += Math.pow(it.getAverageMatcherScore() - average, 2);

        }

        return (Math.sqrt(temp / totalAverages.size()) / (Math.sqrt(totalAverages.size()) * 1.96));

    }

    public void setOutputFile(String outputFile) {

        this.outputFile = outputFile;

    }


    /**
     * Prints the output of trials to STD out
     * OR
     * Saves the output of trials to a file
     */
    public void printResults() {

        if (outputFile != null) {
            writeOutputFile(this.outputFile);

        } else {

            for (ExperimentResults result : trialResults) {

                //result.printTrialParams();

                result.printAggregateScores();
            }
        }
    }


    /**
     * Save a trials into a file
     *
     * @param fileName the file containing results
     */
    private void writeOutputFile(String fileName) {
        File file = new File(fileName);
        try (BufferedWriter output = new BufferedWriter(new FileWriter(file))) {

            System.out.println("writing output file");
            for (ExperimentResults result : trialResults) {

                output.write(String.format("%s%s%n",
                        result.getMangler() + " ",
                        result.getIdfRange().getA()
                                + "-" + result.getIdfRange().getB()));
                // result.getAggregateScores()));

            }

        } catch (IOException e) {
            e.printStackTrace();
        }


    }

    public void aggregateResults() {


        HashMap<String, List<Pair<Integer, Integer>>> temp = new HashMap<>();

        for (ExperimentResults result : trialResults) {

            String key = result.getMangler() + ":" + result.getIdfRange().getA() + "-" + result.getIdfRange().getB();

            if (temp.containsKey(key)) {

                temp.get(key).addAll(result.getAggregateListScores());

            } else {

                temp.put(key, result.getAggregateListScores());
            }


        }


        for (Map.Entry<String, List<Pair<Integer, Integer>>> temporary : temp.entrySet()) {

            HashMap<Integer, Integer> storeResults = new HashMap<>();

            //add any repeating values
            for (Pair<Integer, Integer> histData : temporary.getValue()) {

                if (storeResults.containsKey(histData.getA())) {

                    //TODO port this back to java7
                    // storeResults.replace(histData.getA(), storeResults.get(histData.getA()) + histData.getB());

                } else {

                    storeResults.put(histData.getA(), histData.getB());

                }
            }


            for (Map.Entry<Integer, Integer> out : storeResults.entrySet()) {

                saveAggregate(temporary.getKey(), out.getKey() + ":" + out.getValue() + " ");

            }


        }


    }

    private void saveAggregate(String fileName, String result) {

        try {

            File file = new File(fileName);

            //if file doesnt exists, then create it
            if (!file.exists()) {
                file.createNewFile();
            }

            //true = append file
            FileWriter fileWritter = new FileWriter(file.getName(), true);
            BufferedWriter bufferWritter = new BufferedWriter(fileWritter);
            bufferWritter.write(result);
            bufferWritter.close();


        } catch (IOException e) {
            e.printStackTrace();
        }

    }

}
