package edu.georgetown.gucs.fingerprinter;

import edu.georgetown.gucs.dictionary.Dictionary;
import edu.georgetown.gucs.utility.Token;
import org.apache.xerces.impl.dv.util.Base64;

import java.io.Serializable;
import java.util.List;

/**
 * @author EvanBloomberg
 * @since 5/12/14
 * Utility class that allows for fingerprint to be
 */
public class Fingerprint implements Serializable {
    private String fileName;
    private byte[] fingerprint;
    private int start_byte;
    private int end_byte;
    private String base64Fingerprint;
    private List<Token> tokenList;
    private String sdhash;
    private Dictionary dict;
    private String dictionaryGUID;

    /**
     * Defualt constructor- fingerprints are byte arrays that contain tokenized
     * dictionary information
     *
     * @param fileName    Filename that the fingerprint was created from
     * @param fingerprint an array of bytes that represent the fingerprint
     * @param start_byte  the start location of a fingerprint
     * @param end_byte    the end location of a fingerprint
     */
    public Fingerprint(String fileName, byte[] fingerprint, int start_byte, int end_byte) {
        this.fileName = fileName;
        this.fingerprint = fingerprint;
        this.start_byte = start_byte;
        this.end_byte = end_byte;
    }


    /**
     * Default Constructor- fingerprints are byte arrays that contain tokenized
     * dictionary information
     *
     * @param fileName    Filename that the fingerprint was created from
     * @param tokenList   the list of tokens used to create the fingerprint
     * @param fingerprint an array of bytes that represent the fingerprint
     * @param start_byte  the start location of a fingerprint
     * @param end_byte    the end location of a fingerprint
     */
    public Fingerprint(String fileName, byte[] fingerprint, List<Token> tokenList, int start_byte, int end_byte) {
        this.fileName = fileName;
        this.fingerprint = fingerprint;
        this.start_byte = start_byte;
        this.end_byte = end_byte;
        this.tokenList = tokenList;
    }


    /**
     * Defualt constructor- fingerprints are byte arrays that contain tokenized
     * dictionary information
     *
     * @param fileName    Filename that the fingerprint was created from
     * @param fingerprint an array of bytes that represent the fingerprint
     * @param tokenList   the list of tokens used to create the fingerprint
     * @param dict        the dictionary used to create the fingerprint
     * @param start_byte  the start location of a fingerprint
     * @param end_byte    the end location of a fingerprint
     */
    public Fingerprint(String fileName, byte[] fingerprint, List<Token> tokenList, Dictionary dict, int start_byte, int end_byte) {
        this.fileName = fileName;
        this.fingerprint = fingerprint;
        this.start_byte = start_byte;
        this.end_byte = end_byte;
        this.tokenList = tokenList;
	this.dict = dict;
	this.dictionaryGUID = this.dict.getGUID();
    }

    /**
     * Defualt constructor- fingerprints are byte arrays that contain tokenized
     * dictionary information
     *
     * @param fileName   Filename that the fingerprint was created from
     * @param sdhash     an array of bytes that represent the fingerprint
     * @param tokenList  the list of tokens used to create the fingerprint
     * @param start_byte the start location of a fingerprint
     * @param end_byte   the end location of a fingerprint
     */
    public Fingerprint(String fileName, String sdhash, List<Token> tokenList,
                       int start_byte, int end_byte) {
        this.fileName = fileName;
        this.sdhash = sdhash;
        this.start_byte = start_byte;
        this.end_byte = end_byte;
        this.tokenList = tokenList;
    }


    /**
     * Defualt constructor- fingerprints are byte arrays that contain tokenized
     * dictionary information
     *
     * @param fileName   Filename that the fingerprint was created from
     * @param start_byte the start location of a fingerprint
     * @param end_byte   the end location of a fingerprint
     */
    public Fingerprint(String fileName, int start_byte, int end_byte) {
        this.fileName = fileName;
        this.start_byte = start_byte;
        this.end_byte = end_byte;
    }

    /**
     * Defualt constructor- fingerprints are byte arrays that contain tokenized
     * dictionary information
     *
     * @param fileName          Filename that the fingerprint was created from
     * @param base64Fingerprint a fingerprint that has been encoded in base64
     * @param start_byte        the start location of a fingerprint
     * @param end_byte          the end location of a fingerprint
     */
    public Fingerprint(String fileName, String base64Fingerprint, int start_byte, int end_byte) {
        this.fileName = fileName;
        this.base64Fingerprint = base64Fingerprint;
        this.fingerprint = Base64.decode(base64Fingerprint);
        this.start_byte = start_byte;
        this.end_byte = end_byte;
    }


    /**
     * @param fingerprint the new fingerprint to replace the old fingerprint
     */
    public void setFingerprint(Fingerprint fingerprint) {
        this.fingerprint = fingerprint.getFingerprint();
        this.start_byte = fingerprint.getStartByte();
        this.end_byte = fingerprint.getEndByte();
        this.fileName = fingerprint.getFileName();
    }

    /**
     * @param fileName the name of the file that was used to create a fingerprint
     */
    public void setFileName(String fileName) {
        this.fileName = fileName;
    }

    /**
     * @param start_byte the start byte location where the fingerprint begins
     */
    public void setStartByte(int start_byte) {
        this.start_byte = start_byte;
    }

    /**
     * @param end_byte the end byte location where the fingerprint ends
     */
    public void setEndByte(int end_byte) {
        this.end_byte = end_byte;
    }

    /**
     * @param fingerprint a new byte fingerprint to replace the current byte fingerprint
     */
    public void setFingerprint(byte[] fingerprint) {
        this.fingerprint = fingerprint;
    }

    /**
     * @return a cloned version of the fingerprint as a byte array
     */
    public byte[] getFingerprint() {
        return fingerprint.clone();
    }

    /**
     * @return the byte fingerprint converted to its base64 equivilant
     */
    public String getBase64Fingerprint() {
        return Base64.encode(fingerprint);
    }

    /**
     * @return the start byte in a fingerprint
     */
    public int getStartByte() {
        return start_byte;
    }

    /**
     * @return the ending byte in the fingerprint
     */
    public int getEndByte() {
        return end_byte;
    }

    /**
     * @return the GUID of the dictionary used to create the fingerprint
     */
    public String getCreatingDictionaryGUID(){
	return dictionaryGUID;
    }
    
    /**
     * @return the name of the file used to create the fingerprint
     */
    public String getFileName() {
        return fileName;
    }

    /**
     * @return the string value containing the output from a fingerprint created using sdhash
     */
    public String getSdhash() {
        return sdhash;
    }

    /**
     * @return the number of terms in the dictionary used to create the fingerprint
     */
    public Integer getDictNumTerms() {
        return dict.size();
    }

    /**
     * clears the tokens that were used to create the fingerprint
     */
    public void clearTokenList() {
        if (tokenList.size() > 0) {

            tokenList.clear();

        }
    }

    /**
     * sets the fingerprint dictionary to null
     */
    public void clearDict() {

        dict = null;

    }

    /**
     * @return a list of tokens that were used to create the fingerprint
     */
    public List<Token> getTokenList() {

        return tokenList;

    }

    /**
     * prints all tokens that are contained in the fingerprint
     */
    public void printTokenList() {

        for(Token token : tokenList) {
            System.out.println(token);
        }
    }
}
