package edu.georgetown.gucs.fingerprinter;


import edu.georgetown.gucs.tokenizers.TokenizerList;
import edu.georgetown.gucs.utility.*;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.util.*;
import java.util.logging.Logger;


/**
 * @author Evan Bloomberg
 * @since 4/30/14.
 */
public class SdhashFingerprinter extends Fingerprinter {

    private Map<String, List<Token>> tokenizedFileMap;
    private final Random generator = new Random(System.currentTimeMillis());
    private final String threadName = Thread.currentThread().getName().replaceAll("\\s", "");
    private TokenizerList tokenizers = null;
    private String configFile = "";


    private static final org.slf4j.Logger logger = LoggerFactory.getLogger(SdhashFingerprinter.class);

    //initializes the score threshold to 0
    public SdhashFingerprinter() throws SdTextException {
        super();
    }

    /**
     * Constructor that loads a dictionary and its tokenizers. Generates the fingerprint name, version, unique identifier
     * (GUID), system identifier, and creating program. Sets the fingerprinter output to display all available
     * information.
     *
     * @param dictionaryFilename the filename containing the dictionary to use for this fingerprinter
     */
    public SdhashFingerprinter(String dictionaryFilename) throws SdTextException {
        super(dictionaryFilename);
        try {
            List<String> tokenizerNames = this.dictionary.getTokenizerNames();
            tokenizers = new TokenizerList(tokenizerNames);
        } catch (SdTextException sdtextE) {
            throw sdtextE;
        } catch (Exception e ) {
            ExceptionHandler.handleException(e);
        }
    }

    /**
     * Returns a random number
     *
     * @return a random double based on @value generators value
     */
    private String randomGenerator() {
        return String.valueOf(generator.nextDouble() * 0.5);
    }

    /**
     * Tokenizes a file based on a list of tokenizers that are passed to the function
     *
     * @param newTokenizers a list of tokenizers that will be used to tokenize the file
     */
    public void setTokenizers(TokenizerList newTokenizers) throws SdTextException {

        try {
            tokenizers = new TokenizerList(newTokenizers.getNames());
        } catch (SdTextException sdtextE) {
            throw sdtextE;
        } catch(Exception e) {
            ExceptionHandler.handleException(e);
        }
    }


    /**
     * SetTokenizers sets the tokenizer configuration for the input file
     *
     * @param tokenizerConfigFileName filename of the tokenizer configuration
     */
    public void setTokenizers(String tokenizerConfigFileName) {

        tokenizers = new TokenizerList();
        //tokenizers.readTokenizerConfig(tokenizerConfigFileName);


    }

    //dictionary = initial tokens

    /**
     * computeFingerprint tokenizes a file in order to create a fingerprint
     *
     * @param fileName the file to tokenize
     * @return a list of fingerprints
     */
    @Override
    public List<Fingerprint> computeFingerprint(String fileName) throws SdTextException {
        List<Fingerprint> fingerprint = new ArrayList<>();
        try {
            //dictionary = initial tokens
            //this may be shady @needs testing
            TokenizerList tokenizers = this.dictionary.getTokenizers();

            StringBuilder tokenContainer = new StringBuilder();
            for (Map.Entry<String, List<Token>> splitterEntry : tokenizers.tokenizeFile(fileName).entrySet()) {


                if (splitterEntry.getValue().size() > 0) {

                    for (Token toke : splitterEntry.getValue()) {

                        tokenContainer.append(toke).append(' '); //should this be spaced?

                    }

                    // System.out.println(tokenContainer);

                    try {
                        //stash fingerprint in temp file
                        File tempFile = File.createTempFile("sdhashFpr" + threadName + randomGenerator(), null);

                        try (BufferedWriter bufferedWriter = new BufferedWriter(new FileWriter(tempFile))) {
                            bufferedWriter.write(tokenContainer.toString());
                        }

                        try (InputStream inStream = new ProcessBuilder(Global.sdHashFilePath, tempFile.getCanonicalPath())
                                .redirectErrorStream(true).start().getInputStream();
                             BufferedReader buffer = new BufferedReader(new InputStreamReader(inStream, Global.fileEncoding))) {

                            String line;

                            StringBuilder sdHashOutput = new StringBuilder();
                            while ((line = buffer.readLine()) != null) {
                                sdHashOutput.append(line).append("\n");
                                //    Global.debug("appended+",sdHashOutput.toString() + "\n");
                            }

                            // Global.debug("SdHashOutput", sdHashOutput.toString());
                            List<Token> tokenList = splitterEntry.getValue();

                            fingerprint.add(new Fingerprint(fileName, sdHashOutput.toString(), tokenList, tokenList.get(0).start_byte()
                                    , tokenList.get(tokenList.size() - 1).end_byte()));
                            tempFile.delete();
                        }
                    } catch (IOException e) {
                        ExceptionHandler.handleException(e);
                    }
                }
            }
        } catch (SdTextException sdTextE) {
            throw sdTextE;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
    return fingerprint;
    }


}
