package edu.georgetown.gucs.tokenizers;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Collections;
import java.util.Dictionary;
import java.util.List;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.GZIPInputStream;

import edu.georgetown.gucs.utility.ExceptionHandler;
import edu.georgetown.gucs.utility.Global;
import edu.georgetown.gucs.utility.SdTextException;
import edu.georgetown.gucs.utility.Token;
import org.slf4j.LoggerFactory;

/**
 * Splits contents of a gzipped text file into tokens based on whitespace or by line. Used for {@linkplain Dictionary}
 * creation.
 *
 * @author Clay Shields
 * @author Lindsay Neubauer
 */
public class GzippedFileTokenizer extends FileTokenizer {
    private static final long serialVersionUID = -1100868800293796306L;


    private static final org.slf4j.Logger logger = LoggerFactory.getLogger(GzippedFileTokenizer.class);

    /**
     * Constructor that sets the token creation mode to split based on whitespace
     */
    public GzippedFileTokenizer() {
        this.mode_ = "tokens";
    }

    /**
     * Constructor that sets the token creation mode. If set to "lines" the file is split by line; if set to "tokens" the
     * file is split based on whitespace.
     *
     * @param mode the string mode to split tokens based on whitespace or by line
     */
    public GzippedFileTokenizer(String mode) {
        this.mode_ = mode.trim();
    }

    /**
     * Splits a gzipped text document into tokens
     */
    @Override
    public List<Token> tokenize(String filename) throws SdTextException {

        try {
            if (!filename.contains(".gz")) {
                logger.info("GzippedFileTokenize: Given filename (" + filename
                        + ") does not contain gzip file extension");
                logger.info("tokenize",
                        "Given filename does not contain gzip file extension");
                ExceptionHandler.handleException(new Exception("GzippedFileTokenizer: Given filename (" + filename
                        + ") does not contain gzip file extension"));

            }
            return readFile(filename);
        } catch (SdTextException sdtext) {
            throw sdtext;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
        return null;
    }

    /**
     * Splits a gzipped text document into tokens.
     */
    protected List<Token> readFile(String filename) throws SdTextException {
        BufferedReader buffer_reader = null;
        try {
            // TODO GzippedFileTokenizer: Assumes file is UTF-8 encoded (otherwise, it will spit out garbage)
            buffer_reader = new BufferedReader(new InputStreamReader(new GZIPInputStream(new FileInputStream(filename)),
                    Global.fileEncoding));

            StringBuilder token = new StringBuilder("");
            int character;
            int byte_start = 0;
            int byte_end = 0;

            synchronized (this.tokenVectorMap) {
                this.tokenVectorMap.clear();
                List<Token> tokenVector = Collections.synchronizedList(new Vector<Token>());
                synchronized (tokenVector) {
                    while ((character = buffer_reader.read()) != -1) {
                        if (this.mode_.equalsIgnoreCase("lines")) {
                            if ('\n' == character || '\r' == character || '\f' == character) {
                                if (token.toString().trim().length() > 0) {
                                    tokenVector.add(new Token(token.toString().trim(), byte_start, byte_end));
                                    byte_start = byte_end;
                                }
                                byte_end += Character.charCount(character);
                                token.delete(0, token.length());
                            } else {
                                token.append((char) character);
                                byte_end += Character.charCount(character);
                            }
                        } else {
                            if (Character.isWhitespace(character)) {
                                if (token.toString().trim().length() > 0) {
                                    tokenVector.add(new Token(token.toString().trim(), byte_start, byte_end));
                                    byte_start = byte_end;
                                }
                                byte_end += Character.charCount(character);
                                token.delete(0, token.length());
                            } else {
                                token.append((char) character);
                                byte_end += Character.charCount(character);
                            }
                        }
                    }
                    buffer_reader.close();
                    if (token.toString().trim().length() > 0) {
                        tokenVector.add(new Token(token.toString().trim(), byte_start, byte_end));
                    }
                    this.tokenVectorMap.put("null", tokenVector);
                    return tokenVector;
                }
            }
        } catch (IOException e) {
            ExceptionHandler.handleException(e);
        } finally {
            try {
                if (buffer_reader != null) {
                    buffer_reader.close();
                }
            } catch (IOException e) {
                ExceptionHandler.handleException(e);
            }
        }
        return null;
    }
}