package edu.georgetown.gucs.tokenizers;

import java.util.*;

import edu.georgetown.gucs.utility.*;

/**
 * An ordered list of {@linkplain edu.georgetown.gucs.tokenizers} objects to split a document into tokens and alter the tokens in various
 * ways. The first Tokenizer in this list needs to be able to read directly from a file.
 *
 * @author Clay Shields
 * @author Lindsay Neubauer
 */
public class TokenizerList extends Tokenizer {

    private static final long serialVersionUID = -1191618315036978305L;

    // First tokenizer in this list needs to be able to read directly from a file
    private List<Tokenizer> tokenizers;
    private List<String> names = new ArrayList<>();
    private String splitterString;

    /**
     * Constructor that initializes an empty list of tokenizers
     */
    public TokenizerList() {
        this.tokenizers = new Vector<>();
        this.names = new Vector<>();
    }

    /**
     * Constructor that takes a list of the tokenizer names
     *
     * @param tokenizerNames the string names of the tokenizers to use; the first tokenizer must be able to read from a file
     */
    public TokenizerList(List<String> tokenizerNames) throws SdTextException {
        try {
            this.tokenizers = new Vector<>();
            this.names = new Vector<>();
            addTokenizerNames(tokenizerNames);
        } catch (SdTextException sdtextE) {
            throw sdtextE;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
    }

    /**
     * Enables the mangler with the given settings in the FileManglerTokenizer object in this list
     *
     * @param settings the string containing the mangler settings
     * @param tokens   the string list containing the tokens to put through this mangler
     */
    public void enableMangler(String settings, List<Token> tokens) throws SdTextException {

        try {
            FileManglerTokenizer fmt = (FileManglerTokenizer) this.tokenizers.get(findFileMangler());
            fmt.enableMangler(settings, tokens);
        } catch (SdTextException sdtextE) {
            throw sdtextE;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
    }

    /**
     * Disables the mangler in the FileManglerTokenizer object in this list
     */
    public void disableMangler() throws SdTextException {
        try {
            FileManglerTokenizer fmt = (FileManglerTokenizer) this.tokenizers.get(findFileMangler());
            fmt.disableMangler();
        } catch (SdTextException sdtextE) {
            throw sdtextE;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
    }

    /**
     * Sets the random number generator to use with a FileManglerTokenizer object in this list
     *
     * @param random the random number generator to use if there is a fileManglerTokenizer object in this list
     */
    public void setManglerRNG(Random random) throws SdTextException {
        try {
            FileManglerTokenizer fmt = (FileManglerTokenizer) this.tokenizers.get(findFileMangler());
            fmt.setRNG(random);
        } catch (SdTextException sdtextE) {
            throw sdtextE;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
    }

    /**
     * Finds index of FileManglerTokenizer in this list of tokenizers. Assumes there is a FileManglerTokenizer in this
     * list; otherwise an error occurs.
     *
     * @return the index of the FileManglerTokenizer
     */
    private int findFileMangler() throws SdTextException {
        int index = -1;
        try {
            index = this.names.indexOf("FileManglerTokenizer");
            if (index == -1) {
                ExceptionHandler.handleException(new Exception("TokenizerList: FileManglerTokenizer not found"));
            }
        } catch (SdTextException sdtextE) {
            throw sdtextE;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
        return index;
    }


    /**
     * Sets the splitter for this list
     *
     * @param splitter the string containing the splitter to use; null if not splitting files
     */
    public synchronized void setSplitter(String splitter) {
        this.splitterString = splitter;
    }

    /**
     * Adds a tokenizer to the end of this list of tokenizers. Uses reflection to create the correct object. Allows for
     * runtime specification of tokenizer type.
     *
     * @param tokenizerName the string name of the tokenizer to be added to the end of this list
     */
    public void addTokenizer(String tokenizerName) throws SdTextException {
        try {
            Tokenizer toInsert = (Tokenizer) ObjectBuilder.buildObject(tokenizerName);
            this.tokenizers.add(toInsert);
            this.names.add(tokenizerName);
        } catch (SdTextException sdtextE) {
            throw sdtextE;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
    }


    public List<Tokenizer> getTokenizers() {
        return tokenizers;
    }

    /**
     * @param strTokenizer name of tokenizer that replaces the fileTokenizer
     */
    public synchronized void replaceFileTokenizer(String strTokenizer) throws SdTextException {

        try {
            tokenizers.remove(0);

            Tokenizer toAdd = (Tokenizer) ObjectBuilder.buildObject(strTokenizer);
            tokenizers.add(0, toAdd);
        } catch (SdTextException sdtextE) {
            throw sdtextE;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
    }

    /**
     * Adds tokenizers to the end of this list of tokenizers
     *
     * @param tokenizerNames the list of tokenizers names to use; the first tokenizer must be able to read from a file
     */
    private void addTokenizerNames(List<String> tokenizerNames) throws SdTextException {

        try {
            for (String tokenizers : tokenizerNames) {
                addTokenizer(tokenizers);
            }
        } catch (SdTextException sdtextE) {
            throw sdtextE;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
        // System.out.println(tokenizerNames);
    }


    /**
     * Provides the ordered list of the tokenizer names in this list
     *
     * @return the list of string tokenizer names
     */
    public List<String> getNames() {
        return this.names;
    }


    /**
     * Applies each tokenizer from this list, in order, on the file; the first tokenizer must be able to read from a file
     * and the tokenizers must already be instantiated
     *
     * @param filename the string name of the file to tokenize
     * @return the map of splitter names and their corresponding tokens
     */
    public synchronized Map<String, List<Token>> tokenizeFile(String filename) throws SdTextException {
        try {
            FileTokenizer fileTokenizer = (FileTokenizer) this.tokenizers.get(0);
            fileTokenizer.tokenize(filename);

            Splitter splitter = new Splitter(this.splitterString, fileTokenizer.getTokenVectorMap().get("null"));
            return splitter.tokenizeEachEntry(this.tokenizers.subList(1, this.tokenizers.size()));
        } catch (SdTextException sdtext) {
            throw sdtext;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
        return null;
    }


    //tokenizeString
    public synchronized Map<String, List<Token>> tokenizeString(String str) throws SdTextException {
        try {
            MemoryTokenizer memoryTokenizer = (MemoryTokenizer) this.tokenizers.get(0);
            memoryTokenizer.tokenize(str);

            Splitter splitter = new Splitter(this.splitterString, memoryTokenizer.getTokenVectorMap().get("null"));
            return splitter.tokenizeEachEntry(this.tokenizers.subList(1, this.tokenizers.size()));
        } catch (SdTextException sdtextE) {
            throw sdtextE;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
        //has to be null
        return null;
    }


    @Override
    public String toString() {
        StringBuilder tokenizerNames = new StringBuilder();
        for (String tokenizerName : this.names) {
            tokenizerNames.append(tokenizerName).append(", ");
        }
        tokenizerNames.replace(tokenizerNames.lastIndexOf(","), tokenizerNames.length(), "").trimToSize();
        return tokenizerNames.toString();
    }

}
