package edu.georgetown.gucs.utility;

import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.concurrent.ForkJoinPool;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * Creates a list of files in a directory and allows iteration over all and parts of this list
 *
 * @author Clay Shields
 * @author Lindsay Neubauer
 */
public class FileLister {

    private File directory;
    private static final org.slf4j.Logger logger = LoggerFactory.getLogger(FileLister.class);

    // This is a blocking queue that can be drawn from by threads
    private final LinkedBlockingQueue<File> fileQueue = new LinkedBlockingQueue<>();

    // This is the list of all files found. 
    private final ArrayList<File> fileList = new ArrayList<>();
   

    private boolean loaded = false;
    private final BlockingQueue<File> sharedDirectoryQueue = new LinkedBlockingQueue<>();


    private void init(String directory) throws SdTextException{

	try {
	    this.directory = FileChecker.canRead(directory, "FileLister", "FileLister");
	    
	    recursiveLoadDirectory(directory, fileList);
	}
	catch (SdTextException e){
	    throw e;
	}
    }
    
    /**
     * Constructor that recursively loads files from the given directory and sets the maximum thread count to the number
     * of processors available to the Java virtual machine
     *
     * @param directory the string path to the directory to load
     */
    public FileLister(String directory) throws SdTextException {

	init(directory);
	
    }

    /**
     * Provides the size of the list of all files
     *
     * @return the <code>int</code> number of files
     */
    public ArrayList<File> getFileList(){
	return fileList;
    }

    /**
     * Provides the size of the list of all files
     *
     * @return the <code>int</code> number of files
     */
    public int size() {
        return this.fileList.size();
    }

  /**
     * Provides the size the queue of files
     *
     * @return the <code>int</code> number of files
     */
    public int fileQueueSize() {
        return fileQueue.size();
    }

    /**
     * Provides the boolean indicating if the FileLister has completed loading the file list
     *
     * @return the boolean value indicating if all files have been loaded
     */
    public boolean getLoaded() {
        return this.loaded;
    }


    /**
     * Provides the path to the directory used for the initial list of files
     *
     * @return the string path to the directory used to initially create this list of files
     */
    public File getDirectory() {
        return this.directory;
    }

    /**
     * Decrements the active thread count when a thread finishes processing a directory
     *
     **/

   
    /**
     * Added by EB.
     * Places the fileLister results in a LinkedBlockingQueue
     */
    public void loadQueue() throws SdTextException {

	if (!loaded) {
	    throw new SdTextException("FileLister:loadQueue:Recursive load not complete");
	}

	    
        for (File f : fileList) {
            try {
                fileQueue.put(f);
            } catch (InterruptedException e) {
                ExceptionHandler.handleException(e);
            }
        } 
    }


    /**
     * Retrieves and removes the head of the file queue, waiting if necessary until an element becomes available
     *
     * @return the file at the head of the file queue
     */
    public File takeFile() throws SdTextException {
        try {
            //System.err.println("Thread " + Thread.currentThread().getName() + " taking file " + take);System.err.flush();
            //Global.debug("FileQueueSize", fileQueue.size());
            return this.fileQueue.take();
        } catch (InterruptedException e) {
            logger.error("takeFile", "thread_id = " + Thread.currentThread().getId());
            logger.error("takeFile", Arrays.toString(Thread.currentThread().getStackTrace()));
            ExceptionHandler.handleException(e);
        }
        return null;
    }

  
    /**
     * Returns a list of each of the files in this fileLister, each on a separate line
     */
    public String toString() {
        StringBuilder string = new StringBuilder();
        for (File file : this.fileList) {
            string.append(String.format("%s%n", file.toString()));
        }
        return string.toString();
    }

    /**
     * Recursively loads all the files from the given directory into the list of files used for a dictionary
     *
     * @param directoryName the string path to the directory to load
     */
    private void recursiveLoadDirectory (String directoryName, ArrayList<File> fileList) throws SdTextException{

	//System.err.println("In recursiveload");
	
        /** Multi Thread
         */

        FileListerWorker traverseDirectory = new FileListerWorker(directoryName);

        Global.pool.execute(traverseDirectory);
        while (!traverseDirectory.isDone()) {
            try {
                //Thread.sleep(250);
                TimeUnit.SECONDS.sleep(1);
		//System.err.println("Waiting");
            } catch (InterruptedException e) {
		Global.pool.shutdown();
		throw new SdTextException("FileLister:recursiveLoadDirectory:Threading interuppted");
            }
        }
	//	System.err.println("Pool shutdown");
        
	//System.err.println("Join call");
        fileList.addAll(traverseDirectory.join());
	loaded = true;
	//System.err.println("Past join");
    }


    public ArrayList<File> getFileListerList() {
        return fileList;
    }

 
    public LinkedBlockingQueue<File> getFileQueue() {

        if (loaded) {
            return fileQueue;
        }
        return null;
    }

  /**
     * Adds a file to the end of the queue as a marker for work being done
     *
     * @param poison the file to put at the tail of the file queue
     */
    public synchronized void putPoison(File poison) throws SdTextException {
        try {
            this.fileQueue.put(poison);
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
    }
    

    /**
     * Populates the dictionary and sample lists, waiting for all files to be loaded before randomly assigning files to
     * one of the lists. Dictionary and sample lists do not share any files. If this method is never called all files are
     * assigned to the dictionary list.
     *
     * @param dictionarySize the <code>double</code> number of files to include in the dictionary list
     * @param sampleSize     the <code>double</code> number of files to include in the sample list
     * @return a pair that contains the dictionarylist and the sample list
     */
    synchronized public Pair getSublists(double dictionarySize, double sampleSize) throws Exception {

        while (!this.getLoaded()) {
            // Wait for the list to be loaded before copying it
            try {
                // Not done yet
                Thread.sleep(100);
                //wait();
                logger.info("[getLoaded = when filelister is ready]  sleeping");
            } catch (InterruptedException e) {
                ExceptionHandler.handleException(e);
            }
        }


        if (dictionarySize < 0 || sampleSize < 0) {
            ExceptionHandler.handleException(new Exception("FileLister: Negative size parameter(s): dictionarySize = " + dictionarySize + "  sampleSize = " + sampleSize));
        }


        int dictionaryCount = (int) ((dictionarySize >= 1) ? dictionarySize
                : ((dictionarySize * 10 * this.fileList.size()) + .5));
        int sampleCount = (int) ((sampleSize >= 1) ? sampleSize : ((sampleSize * 10 * this.fileList.size()) + .5));

        logger.info("dictionaryCount= {}", dictionaryCount);
        logger.info("sampleCount= {}", sampleCount);

        //    logger.log(Level.INFO, "[dictionaryCount] " + dictionaryCount + " > " + (fileList.size() - sampleCount));
        //   logger.log(Level.INFO, "[sampleCount] " + sampleCount + " > " + (fileList.size() - dictionaryCount));
        if (dictionaryCount > this.fileList.size() - sampleCount
                || sampleCount > this.fileList.size() - dictionaryCount) {
            ExceptionHandler.handleException(new Exception(String.format("%s%n  %s%n  %s%n  %s%n",
                    "FileLister: Incompatible parameters: dictionary files + sample files <= total directory files",
                    "Total directory files: " + this.fileList.size(), "Requested number of dictionary files: "
                            + dictionaryCount, "Requested number of sample files: " + sampleCount
            )));

            logger.error("[SEVERE] Dictionary count > fileListSize - dictionaryCount");
        }

        logger.info("[fileList size] {}", fileList.size());
        Collections.shuffle(this.fileList);

        ArrayList<File> outputDictionaryList = new ArrayList<>();
        ArrayList<File> outputSampleList = new ArrayList<>();
        outputDictionaryList.addAll(this.fileList.subList(0, dictionaryCount));
        outputSampleList.addAll(this.fileList.subList(dictionaryCount, dictionaryCount + sampleCount));

        logger.info("[outputDictionaryList size] {}", outputDictionaryList.size());
        logger.info("[outputSampleList size] {} ", outputSampleList.size());

        //prints to std out when sublist completes
        logger.info("FileLister: createSublists: completed");

        return (new Pair<>(outputDictionaryList, outputSampleList));

    }
}
