package enums;


/**
 * Provides the different programs available within SDText (Similarity Digest Text)
 *
 * @author Lindsay Neubauer
 */
public enum CommandLineHelp {

    /**
     * Contains general help information for running SDText via the command line
     */
    HELP(
            "SDText",
            String.format("%s %n%s %n%s %n%s", "Usage: java -jar sdtext.jar PROGRAM [OPTION]...",
                    "Run PROGRAM with specified OPTION(s).", "Example: sdtext TokenizeFile -i document.txt -c tokenizers.xml",
                    "Example: sdtext TokenizeFile --input document.txt --config tokenizers.xml"),
            String
                    .format(
                            "%n%s%n %n%s %n%s %n%s %n%s %n%s %n%s %n%s %n%s %n%s %n%s %n%s %n%s %n%s %n%s",
                            "OPTIONS:",
                            "-i, --input                 filename to read for input",
                            "-c, --config                filename of configuration file",
                            "-o, --output                filename to write output",
                            "-d, --dictionary            filename containing dictionary",
                            "-f, --fingerprint           filename containing fingerprint",
                            "-t, --targetFingerprint     the target fingerprint for scoreFingerprints",
                            "-g, --sourceFingerprint     the source fingerprint for scoreFingerprints",
                            "-p, --directory             path to a directory",
                            "-m, --matcher               name of matcher to use",
			    "-l, --min-idf               minimum normalized IDF (0.0 - 1.0) to consider in dictionary, default = 0.0",
                            "-h, --max-idf               maximum normalized IDF (0.0 - 1.0) to consider in dictionary, default = 1.0",
                            "-s, --min-score             minimum score (0-100) needed to output, default = 0",
                            "-q, --debug                 optional: Specify level for logging to log named sdtext.log. Logging is off by default.",
                            "-h, --help                  display help menu")),

    /**
     * Contains information for running the <code>TokenizeFile</code> program via the command line.
     * <code>TokenizeFile</code> tokenizes a text file and prints the resulting tokens to the the screen.
     */
    TOKENIZEFILE("TokenizeFile", "Tokenizes a text file and prints the resulting tokens to the the screen", String
            .format("%s %n%s %n%s", "-i, --input          filename of document to split into tokens",
                    "-c, --config         optional: filename of tokenizer configuration file, default = doc/configuration_files/tokenizers_config.xml",
                    "-q, --debug          optional: Specify level for logging to log named sdtext.log. Logging is off by default.")),

    /**
     * Contains information for running the <code>ArabicFileTokenizer</code> program via the command line.
     * <code>ArabicFileTokenizer</code> tokenizes an Arabic text file and prints the resulting tokens to the the screen.
     */
    ARABICFILETOKENIZER("ArabicFileTokenizer",
            "Tokenizes an Arabic text file and prints the resulting tokens to the the screen", String.format("%s %n%s",
            "-i, --input          filename of Arabic language document to split into tokens",
            "-q, --debug          optional: Specify level for logging to log named sdtext.log. Logging is off by default.")),

    /**
     * Contains information for running the <code>ChineseFileTokenizer</code> program via the command line.
     * <code>ChineseFileTokenizer</code> tokenizes a Chinese text file into single characters and using a probabilistic
     * model to group characters and prints both sets of resulting tokens to the the screen.
     */
    CHINESEFILETOKENIZER(
            "ChineseFileTokenizer",
            "Tokenizes a Chinese text file first into single characters and second using a probabilistic model to group characters and prints both sets of resulting tokens to the the screen",
            String.format("%s %n%s",
                    "-i, --input          filename of Chinese language document to split into tokens",
                    "-q, --debug          optional: Specify level for logging to log named sdtext.log. Logging is off by default.")),

    /**
     * Contains information for running the <code>CreateDictionary</code> program via the command line.
     * <code>CreateDictionary</code> creates a dictionary from the specified directory using the provided tokenizers.
     */
    CREATEDICTIONARY("CreateDictionary",
            "Creates a dictionary from the specified directory using the provided tokenizers", String.format("%s %n%s %n%s %n%s",
            "-p, --directory      directory to create the dictionary from",
            "-c, --config         optional: filename of the tokenizer configuration file, default = doc/configuration_files/tokenizers_config.xml",
            "-d, --dictionary     filename to write the dictionary, default = standard output",
            "-q, --debug          optional: Specify level for logging to log named sdtext.log. Logging is off by default.")),

    /**
     * Contains information for running the <code>CreateDictionary</code> program via the command line.
     * <code>CreateDictionary</code> creates a dictionary from the specified directory using the provided tokenizers.
     */
    CREATEDICTTRIM("CreateDictTrim",
            "Creates a dictionary from the specified directory using the provided tokenizers", String.format("%s %n%s %n%s %n%s %n%s %n%s",
            "-p, --directory      directory to create the dictionary from",
            "-d, --dictionary     filename containing the dictionary that will be created",
            "-c, --config         optional: filename of the tokenizer configuration file, default = doc/configuration_files/tokenizers_config.xml",
            "-l, --min-idf        minimum normalized IDF (0.0 - 1.0) to consider in dictionary, default = 0.0",
            "-h, --max-idf        maximum normalized IDF (0.0 - 1.0) to consider in dictionary, default = 1.0",
             "-q, --debug          optional: Specify level for logging to log named sdtext.log. Logging is off by default.")),

    /**
     * Contains information for running the <code>ShowDictionaryStatistics</code> program via the command line.
     * <code>ShowDictionaryStatistics</code> prints statistics for a dictionary.
     */
    SHOWDICTIONARYSTATISTICS("ShowDictionaryStatistics", "Prints statistics for a dictionary", String.format("%s %n%s",
            "-d, --dictionary     filename containing the dictionary to show",
            "-q, --debug          optional: Specify level for logging to log named sdtext.log. Logging is off by default.")),

    /**
     * Contains information for running the <code>ShowDictionaryTokens</code> program via the command line.
     * <code>ShowDictionaryTokens</code> prints dictionary tokens and their frequencies, IDFs, and normalized IDFs.
     */
    SHOWDICTIONARYTOKENS("ShowDictionaryTokens",
            "Prints dictionary tokens and their frequencies, IDFs, and normalized IDFs", String.format("%s %n%s",
            "-d, --dictionary     filename containing the dictionary to show",
            "-q, --debug          optional: Specify level for logging to log named sdtext.log. Logging is off by default.")),

    /**
     * Contains information for running the <code>TrimDictionary</code> program via the command line.
     * <code>TrimDictionary</code> trims a dictionary by removing any token that is outside a range of normalized IDFs.
     */
    TRIMDICTIONARY("TrimDictionary",
            "Trims a dictionary by removing any token that is outside a range of normalized IDFs", String.format(
            "%s %n%s %n%s %n%s %n%s", "-d, --dictionary     filename containing the dictionary to trim",
            "-l, --min-idf        minimum normalized IDF (0.0 - 1.0) to keep in trimmed dictionary, default = 0.0",
            "-h, --max-idf        maximum normalized IDF (0.0 - 1.0) to keep in trimmed dictionary, default = 1.0",
            "-o, --output         filename to write the trimmed dictionary",
            "-q, --debug          optional: Specify level for logging to log named sdtext.log. Logging is off by default.")),

    /**
     * Contains information for running the <code>TokenizeFile</code> program via the command line.
     * <code>TokenizeFile</code> tokenizes a text file and prints the resulting tokens to the the screen.
     */
    BITVECTORFINGERPRINTER("BitVectorFingerprinter",
            "Creates a bit vector fingerprint representing the presence or absence of terms in a particular dictionary",
            String.format("%s %n%s %n%s %n%s %n%s", "-i, --input          filename of the document to fingerprint",
                    "-d, --dictionary     filename containing the dictionary to use",
                    "-c, --config         optional: filename containing the output configuration file",
                    "-o, --output         optional: filename to write the XML output",
                    "-q, --debug          optional: Specify level for logging to log named sdtext.log. Logging is off by default.")),

    /**
     * Contains information for running the <code>ScoreFingerprints</code> program via the command line.
     * <code>ScoreFingerprints</code> compares two XML fingerprints and provides a score for their degree of similarity.
     */
    SCOREFINGERPRINTS("ScoreFingerprints",
            "Compares two XML fingerprints and provides a score for their degree of similarity", String.format(
            "%s %n%s %n%s %n%s",
            "-m, --matcher              matcher name to score the fingerprints, default = CosineSimilarityFingerprintMatcher",
            "-t, --targetFingerprint    filename containing fingerprint",
            "-g, --sourceFingerprint    filename containing fingerprint",
            "-q, --debug                optional: Specify level for logging to log named sdtext.log. Logging is off by default.")),

    /**
     * Contains information for running the <code>ExtractFingerprintDictionary</code> program via the command line.
     * <code>ExtractFingerprintDictionary</code> extracts a dictionary from a fingerprint digest and writes it to a new
     * dictionary file.
     */
    EXTRACTFINGERPRINTDICTIONARY("ExtractFingerprintDictionary",
            "Extracts a dictionary from a fingerprint digest and writes it to a new dictionary file", String.format(
            "%s %n%s %n%s",
            "-f, --fingerprint    filename containing the fingerprint and its dictionary to extract",
            "-o, --output         optional: filename to write the extracted dictionary, default = standard output",
            "-q, --debug          optional: Specify level for logging to log named sdtext.log. Logging is off by default.")),

    /**
     * Contains information for running the <code>CompareFingerprint</code> program via the command line.
     * <code>CompareFingerprint</code> outputs a list of files and their scores, comparing a given fingerprint to a
     * directory of files.
     */
    COMPAREFINGERPRINT("CompareFingerprint",
            "Outputs a list of files and their scores, comparing a given fingerprint to a directory of files", String.format(
            "%s %n%s %n%s %n%s %n%s %n%s %n%s ",
            "-p, --directory         directory path to compare",
            "-f, --fingerprint       filename containing fingerprint",
            "-d, --dictionary        optional: filename containing the dictionary to use; only needed if the fingerprint "
                    + "file does not contain a full dictionary or a link to a dictionary in the file system",
            "-m, --matcher           optional: matcher name to make comparison, default = CosineSimilarityFingerprintMatcher",
            "-o, --output            optional: filename to write output",
            "-s, --min-score         optional: minimum score (0-100) needed to output a file, default = 0",
            "-q, --debug             optional: Specify level for logging to log named sdtext.log. Logging is off by default.")),

    COMPAREDIRECTORY("CompareDirectory",
            "Outputs a list of files and their scores using n^2 comparison", String.format(
            "%s %n%s %n%s %n%s %n%s %n%s %n%s",
            "-p, --directory         directory path to compare",
            "-d, --dictionary        filename containing the dictionary to use; only needed if the fingerprint "
                    + "file does not contain a full dictionary or a link to a dictionary in the file system",
            "-o, --output            optional: filename to write output",
            "-s, --min-score         optional: minimum score (0-100) needed to output a file, default = 0",
            "-m, --matcher           optional: matcher name to make comparison GoogleFingerprintMatcher" +
                    " or (default) CosineSimilarityFingerprintMatcher",
            "-f  --fingerprint       optional: filename containing fingerprint, default = BitVectorFingerprinter",
            "-q, --debug             optional: Specify level for logging to log named sdtext.log. Logging is off by default.")),

    /**
     * Contains information for running the <code>Experiment</code> program via the command line. <code>Experiment</code>
     * outputs results from multiple runs with different experiment settings and optionally stores them in a database.
     */
    EXPERIMENT("Experiment",
            "Outputs results from multiple runs with different experiment settings", String.format("%s %n%s %n%s %n%s",
            "-c, --config         filename of the experiment configuration file",
            "-o, --output         optional: filename to write output",
            "-g, --graph          optional: filename to write graph",
            "-q, --debug          optional: Specify level for logging to log named sdtext.log. Logging is off by default.")),

    /**
     * Contains information for running the <code>Experiment</code> program via the command line. <code>Experiment</code>
     * outputs results from multiple runs with different experiment settings and optionally stores them in a database.
     */
    SDHASHFINGERPRINTER("SdhashFingerprinter", "Creates a SDHash fingerprint representing the presence or absence of terms in a particular dictionary", String.format("%s %n%s %n%s %n%s",
            "-d, --dictionary     filename containing the dictionary to use",
            "-s, --min-score         optional: minimum score (0-100) needed to output a file, default = 0",
            "-c, --config         optional: Tokenizer Configuration file, default = doc/configuration_files/tokenizers_config.xml",
            "-q, --debug          optional: Specify level for logging to log named sdtext.log. Logging is off by default.")),

    /**
     * Contains information for running the <code>Experiment</code> program via the command line. <code>Experiment</code>
     * outputs results from multiple runs with different experiment settings and optionally stores them in a database.
     */
    FINGERPRINTSTOTEXT("FingerprintsToText", "Compare text to a set of files of fingerprints", String.format("%s %n%s %n%s %n%s %n%s %n%s %n%s %n%s",
            "-d, --dictionary     filename containing the dictionary to use",
            "-g, --sourceFingerprint     string to use for comparison (use this OR -i)",
            "-i, --input          Input file of strings (use this OR -g)",
    	    "-p, --directory	  Path to directory containing fingerprint files (use this OR -f)",
	    "-f, --fingerprint	  File containing paths to fingerprint files to read (use this OR -p)",
	    "-m, --matcher           optional: matcher name to use, CosineSimilarityMatcher by default",
            "-s, --min-score         optional: minimum score (0-100) needed to output a file, default = 0",	
	    "-q, --debug          optional: Specify level for logging to log named sdtext.log. Logging is off by default.")),

    /**
     * Contains information for running the <code>Experiment</code> program via the command line. <code>Experiment</code>
     * outputs results from multiple runs with different experiment settings and optionally stores them in a database.
     */
    COMPAREFINGTODIR("CompareFingToDirectory", "Creates a fingerprint from text and compares it to a list of fingerprints", String.format("%s %n%s %n%s %n%s %n%s",
            "-f, --fingerprint    filename containing the dictionary to use",
            "-p, --directory      directory path to compare",
            "-d, --dictionary     dictionary that was used to create the fingerprints ",
            "-s, --min-score      optional: Input file to run the fingerprinter on",
            "-m, --matcher        optional: matcher name to score the N2 fingerprints, default = CosineSimilarityFingerprintMatcher ",
            "-q, --debug          optional: Specify level for logging to log named sdtext.log. Logging is off by default.")),

    /**
     * Contains information for running the <code>Experiment</code> program via the command line. <code>Experiment</code>
     * outputs results from multiple runs with different experiment settings and optionally stores them in a database.
     */
    TESTDICTIONARY("TestDictionary", "Compares two dictionaries to determine if tokens, IDF range is the same", String.format("%s %n%s",
            "-d, --dictionary    filename containing the dictionary to use",
            "-p, --dictionary2   filename containing the dictionary to use")),

    /**
     * Contains information for running the <code>Experiment</code> program via the command line. <code>Experiment</code>
     * outputs results from multiple runs with different experiment settings and optionally stores them in a database.
     */
    TESTFINGERPRINT("TestFingerprint", "Compares two Fingerprints to determine if they are the same", String.format("%s %n%s",
            "-f, --fingerprint   filename containing the fingerprint to test",
														    "-d, --dictionary    [optional] filename containing the dictionary to use"));


    public final String name;
    public final String use;
    public final String options;

    /**
     * Stores information about each program
     *
     * @param name    the string name of the program
     * @param use     the string explaining the purpose of the program
     * @param options the string specifying the command line options for the program
     */
    CommandLineHelp(String name, String use, String options) {
        this.name = name;
        this.use = use;
        this.options = options;
    }
}
